@@ -24,7 +24,6 @@ modules = [
     "artstation",
     "aryion",
     "bbc",
-    "bcy",
     "behance",
     "blogger",
     "bunkr",
@@ -50,7 +49,6 @@ modules = [
     "gelbooru",
     "gelbooru_v01",
     "gelbooru_v02",
-    "gfycat",
     "gofile",
     "hbrowse",
     "hentai2read",
@@ -85,7 +83,6 @@ modules = [
     "lensdump",
     "lexica",
     "lightroom",
-    "lineblog",
     "livedoor",
     "luscious",
     "lynxchan",
